/**************************************************************************************************

Copyright (c) 2007   Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#include  "mitaka.h"
#include  "mtk_spacecraft.h"
#include  "mtk_spacecraft_model.h"
#include  "mtk_directory.h"
#include  "../common/graphics/3ds_loader.h"
#include  "mtk_gl_common.h"
#include  "mtk_font.h"
#include  "../common/astro/astro_const.h"
#include  "mtk_planet_position.h"
#include  "mtk_switches.h"
#include  "mtk_objects.h"
#include  "mtk_label.h"
#include  "mtk_colors.h"
#include  "mtk_solar_system.h"
#include  "mtk_calc_alpha.h"


//***
//--  Externs
void	set_light_from_sun(const vector3d& pos);





//****  \̔a  *****
const float Spacecraft_R = 100.0 * _1km_pc;




//-----------
//  Objects
//-----------
static spacecraft  vSpacecraft[5];
static vector3d vSpacecraftPosition[5];
//---  3D Models  ---
static glModel3D  mdlPioneer, mdlVoyager, mdlCassini;


void
init_spacecraft()
{
	directory  dir = get_directory(DIR_SPACECRAFT);

	//-- Load models
	load_3DS_file( dir.get_path("Pioneer.3ds"), mdlPioneer);
	load_3DS_file( dir.get_path("Voyager.3ds"), mdlVoyager);
	load_3DS_file( dir.get_path("Cassini.3ds"), mdlCassini);

	mdlPioneer.fit_size(1.0);
	mdlVoyager.fit_size(1.0);
	mdlCassini.fit_size(1.0);
	mdlCassini.set_r0( vector3f(0,0,0.5) );

	//-- init spacecraft
	vSpacecraft[0].set( "Pioneer 10",	"pCIjAPO",	&mdlPioneer);
	vSpacecraft[1].set( "Pioneer 11",	"pCIjAPP",	&mdlPioneer);
	vSpacecraft[2].set( "Voyager I",	"{CW[P",		&mdlVoyager);
	vSpacecraft[3].set( "Voyager II",	"{CW[Q",		&mdlVoyager);
	vSpacecraft[4].set( "Cassini",		"JbV[j",			&mdlCassini);

	//-- load path
	vSpacecraft[0].load_path( dir.get_path("Pioneer10.pth") );
	vSpacecraft[1].load_path( dir.get_path("Pioneer11.pth") );
	vSpacecraft[2].load_path( dir.get_path("Voyager_I.pth") );
	vSpacecraft[3].load_path( dir.get_path("Voyager_II.pth") );
	vSpacecraft[4].load_path( dir.get_path("Cassini.pth") );
}


//------------
//  Register
//------------
void
register_spacecraft()
{
	const char key[][50] = {
		"Pioneer10",
		"Pioneer11",
		"VoyagerI",
		"VoyagerII",
		"Cassini"
	};

	bilingual szName[] = {
		{ "Pioneer 10",	"pCIjAPO" },
		{ "Pioneer 11",	"pCIjAPP" },
		{ "Voyager I",	"{CW[P" },
		{ "Voyager II",	"{CW[Q" },
		{ "Cassini",	"JbV[j" }
	};


	mtk_object  obj;

	for (int i=0; i<5; i++) {
		//  register object
		obj.init(OBJ_TYPE_SPACECRAFT, i, 0.0f, true);
		int obj_idx = register_object(obj, key[i]);
	
		// register label
		float thc = 5 * D2R;

		char str[256];
		sprintf(str, "%sLabel", key[i]);
		add_label( szName[i].text_J, szName[i].text_E, obj_idx,
			MTK_COLOR_NAME_SPACECRAFT, 1.0f, 0.9*get_object_name_size(), Spacecraft_R, thc, true, str );
	}
}



//------------------
//  Landing radius
//------------------
double
get_landing_radius_spacecraft(int idx)
{
	return  Spacecraft_R;		//****
}



//----------------
//  Prepare draw
//----------------
bool
prepare_draw_spacecraft(const mtk_draw_struct& ds, int idx, double* pRb, double* pRh)
{
	//---  Visible
	if  (!bSpacecraft)  return false;
	if (ds.JD < vSpacecraft[idx].get_t1())  return false;

	//---  Radius
	*pRb = *pRh = Spacecraft_R;		//****

	return  true;
}



//---------
//  Scale
//---------
double
get_scale_spacecraft(int idx)
{
	return  30.0 * Spacecraft_R;	//****
}



//---------
//  Label
//---------
bool
is_spacecraft_label_visible(int idx)
{
	if (!bSpacecraft)  return false;

	//****
	return  test_disp_switch(DISPSW_PLANET, DISPM_NAME);
}


float
get_spacecraft_label_alpha(const mtk_draw_struct& ds, int idx)
{
	const spacecraft& spccrf = vSpacecraft[idx];
	if (ds.JD < spccrf.get_t1())  return 0.0f;

	const vector3d pos  = get_spacecraft_position( idx );
	const vector3d rpos = pos - ds.camera.get_pos();

	float Rv;
	if (!spccrf.get_draw_path_mode()) {
		// O\ĂȂ́A߂Â̂ݖO\
		Rv = Spacecraft_R;
	} else {
		// O\́AfƓ
		Rv = norm(pos);
	}

	float  Dth = 2.0f * Rv / rpos.norm();
	return  calc_alpha_by_apparent_diameter( Dth );
}



//--------
//  Info
//--------
void
make_object_info_spacecraft(const mtk_draw_struct& ds, int idx, std::vector<std::string>& text)
{
	bilingual bszName = vSpacecraft[idx].get_name();

	if (ds.lang == LANG_ENG) {
		text.push_back( bszName.text_E );
		text.push_back( "Type: Spacecraft" );
	} else {
		text.push_back( bszName.text_J );
		text.push_back( "ށFT@" );
	}
}





//--------
//  Name
//--------
bilingual
get_spacecraft_name(int idx)
{
	return  vSpacecraft[idx].get_name();
}



//------------
//  Position
//------------
void
update_spacecraft_position(const double& t)
{
	for (int i=0; i<5; i++) {
		vSpacecraftPosition[i] = vSpacecraft[i].get_pos(t);
	}
}

vector3d
get_spacecraft_position(int idx)
{
	return  vSpacecraftPosition[idx];
}


//-------------
//  Draw Path
//-------------
bool
get_spacecraft_path_mode(int idx)
{
	return  vSpacecraft[idx].get_draw_path_mode();
}

void
set_spacecraft_path_mode(int idx, bool bFlg)
{
	vSpacecraft[idx].set_draw_path_mode(bFlg);
}





//---------------------
//  T@̃f̕\
//---------------------
void
//draw_a_spacecraft(const mtk_draw_struct& ds, const spacecraft& spccrf)
draw_a_spacecraft(const mtk_draw_struct& ds, int idx, const vector3d& rpos, const double& scl)
{
	const spacecraft& spccrf = vSpacecraft[idx];
	const vector3d  pos = spccrf.get_pos(ds.JD);		// T@̈ʒu

	// ̐ݒ
	set_light_from_sun(pos);

	// T@n̕Ɍs
	const vector3d  posE = get_planet_position(PLNT_EARTH);	// n̈ʒu
	vector3d nz = -normalize( posE - pos );
	vector3d nu = vector3d(0,0,1);
	vector3d nx = normalize( cross_product(nu, nz) );
	vector3d ny = normalize( cross_product(nz, nx) );

	double MR[16];
	MR[0] = nx.x;
	MR[1] = nx.y;
	MR[2] = nx.z;
	MR[3] = 0.0;

	MR[4] = ny.x;
	MR[5] = ny.y;
	MR[6] = ny.z;
	MR[7] = 0.0;

	MR[8]  = nz.x;
	MR[9]  = nz.y;
	MR[10] = nz.z;
	MR[11] = 0.0;

	MR[12] = 0.0;
	MR[13] = 0.0;
	MR[14] = 0.0;
	MR[15] = 1.0;

	vector3d rr = rpos;

	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glPushMatrix();
		glTranslated(rr.x,rr.y,rr.z);
		glMultMatrixd(MR);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		glEnable(GL_NORMALIZE);
		glEnable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);

		const float R = scl * 100.0 * _1km_pc;
		glScalef(R, R, R);


		spccrf.draw_model();

	glPopMatrix();
	glPopAttrib();
}


void
draw_spacecraft_path(const mtk_draw_struct& ds)
{
	const vector3d camera_pos = ds.camera.get_pos();
	for (int i=0; i<5; i++) {
		const spacecraft& spccrf = vSpacecraft[i];
		const vector3d pos  = spccrf.get_pos(ds.JD);
		const vector3d rpos = pos - ds.camera.get_pos();
		float  Rv = norm(pos);
		float  Dth = 2.0f * Rv / rpos.norm();
		float  af = calc_alpha_by_apparent_diameter( Dth );

		if (af <= 0.01f)  continue;
		RGBf  col;
		if (ds.JD <= spccrf.get_t2())	{ col.set(1,1,0); }
		else							{ col.set(0.7,0.5,0); }
		spccrf.draw_path(ds, ds.JD, camera_pos, col, af);
	}
}




